use sp_runtime::curve::PiecewiseLinear;

pallet_staking_reward_curve::build! {
    const I_NPOS: PiecewiseLinear<'static> = curve!(
        min_inflation: 0_025_000,
        max_inflation: 0_100_000,
        ideal_stake: 0_500_000,
        falloff: 0_050_000,
        max_piece_count: 40,
        test_precision: 0_005_000,
    );
}

#[test]
fn npos_curve_is_sensible() {
    const YEAR: u64 = 365 * 24 * 60 * 60 * 1000;

    // check maximum inflation.
    // not 10_000 due to rounding error.
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 0, 100_000u64, YEAR).1,
        9_993
    );

    //super::I_NPOS.calculate_for_fraction_times_denominator(25, 100)
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 0, 100_000u64, YEAR).0,
        2_498
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 5_000, 100_000u64, YEAR).0,
        3_248
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 25_000, 100_000u64, YEAR).0,
        6_246
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 40_000, 100_000u64, YEAR).0,
        8_494
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 50_000, 100_000u64, YEAR).0,
        9_993
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 60_000, 100_000u64, YEAR).0,
        4_379
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 75_000, 100_000u64, YEAR).0,
        2_733
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 95_000, 100_000u64, YEAR).0,
        2_513
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 100_000, 100_000u64, YEAR).0,
        2_505
    );

    const DAY: u64 = 24 * 60 * 60 * 1000;
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 25_000, 100_000u64, DAY).0,
        17
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 50_000, 100_000u64, DAY).0,
        27
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 75_000, 100_000u64, DAY).0,
        7
    );

    const SIX_HOURS: u64 = 6 * 60 * 60 * 1000;
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 25_000, 100_000u64, SIX_HOURS).0,
        4
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 50_000, 100_000u64, SIX_HOURS).0,
        7
    );
    assert_eq!(
        super::compute_total_payout(&I_NPOS, 75_000, 100_000u64, SIX_HOURS).0,
        2
    );

    const HOUR: u64 = 60 * 60 * 1000;
    assert_eq!(
        super::compute_total_payout(
            &I_NPOS,
            2_500_000_000_000_000_000_000_000_000u128,
            5_000_000_000_000_000_000_000_000_000u128,
            HOUR
        )
        .0,
        57_038_500_000_000_000_000_000
    );
}

